<?php

namespace LakeDrops\DrupalConfig;

use Composer\Composer;
use Composer\Installers\Installer;
use Composer\IO\IOInterface;
use Composer\Plugin\PluginInterface;

/**
 * Composer plugin for handling drupal scaffold.
 */
class Plugin implements PluginInterface {

  /**
   * The installer.
   *
   * @var \LakeDrops\DrupalConfig\Installer
   */
  private Installer $installer;

  /**
   * {@inheritdoc}
   */
  public function activate(Composer $composer, IOInterface $io): void {
    $this->installer = new Installer($io, $composer);
    $composer->getInstallationManager()->addInstaller($this->installer);
    $extra = $composer->getPackage()->getExtra();

    if ($extra['lakedrops-default'] ?? FALSE) {
      $default = [
        'lakedrops-config-file' => 'https://gitlab.lakedrops.com/composer/plugin/drupal-environment/-/raw/main/defaults/config.json',
        'lakedrops-curated-file' => 'https://gitlab.lakedrops.com/composer/plugin/drupal-environment/-/raw/main/defaults/curated.json',
        'lakedrops-extra-drupal-scaffold-file' => 'https://gitlab.lakedrops.com/composer/plugin/drupal-environment/-/raw/main/defaults/drupal-scaffold.json',
        'lakedrops-extra-drupal-libraries-file' => 'https://gitlab.lakedrops.com/composer/plugin/drupal-environment/-/raw/main/defaults/drupal-libraries.json',
        'lakedrops-extra-installer-types-file' => 'https://gitlab.lakedrops.com/composer/plugin/drupal-environment/-/raw/main/defaults/installer-types.json',
        'lakedrops-extra-installer-paths-file' => 'https://gitlab.lakedrops.com/composer/plugin/drupal-environment/-/raw/main/defaults/installer-paths.json',
      ];
      foreach ($default as $key => $value) {
        if (!isset($extra[$key])) {
          $extra[$key] = $value;
        }
      }
    }
    if (isset($extra['lakedrops-config-file']) || isset($extra['lakedrops-curated-file'])) {
      if (isset($extra['lakedrops-config-file'])) {
        foreach ($this->readJson('config', $extra['lakedrops-config-file'], $io) as $key => $value) {
          $composer->getConfig()
            ->getConfigSource()
            ->addConfigSetting($key, $value);
        }
      }
      if (isset($extra['lakedrops-curated-file'])) {
        $audit = $composer->getConfig()->get('audit');
        $audit['ignore'] = $this->readJson('audit ignore', $extra['lakedrops-curated-file'], $io);
        $composer->getConfig()
          ->getConfigSource()
          ->addConfigSetting('audit', $audit);
      }
    }

    $extras = [
      'drupal-libraries',
      'drupal-scaffold',
      'installer-types',
      'installer-paths',
    ];
    $changed = FALSE;
    foreach ($extras as $item) {
      $key = 'lakedrops-extra-' . $item . '-file';
      if (isset($extra[$key])) {
        if (!isset($extra[$item])) {
          $extra[$item] = [];
        }
        if ($item === 'installer-paths') {
          $installer = new DrupalInstaller($composer->getPackage(), $composer, $io);
          foreach ($installer->getLocations('drupal') as $type => $path) {
            $extra[$item][$path] = ['type:drupal-' . $type];
          }
        }
        $extra[$item] += $this->readJson($item, $extra[$key], $io);
        $changed = TRUE;
      }
    }
    if ($changed) {
      $composer->getPackage()->setExtra($extra);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deactivate(Composer $composer, IOInterface $io): void {
    $composer->getInstallationManager()->removeInstaller($this->installer);
  }

  /**
   * {@inheritdoc}
   */
  public function uninstall(Composer $composer, IOInterface $io): void {
  }

  /**
   * Helper function to read and decode default settings from a file.
   *
   * @param string $type
   *   The type of information for status message.
   * @param string $filename
   *   The filename from where to read the data.
   * @param \Composer\IO\IOInterface $io
   *   The io service for messages.
   *
   * @return array
   *   The read data.
   *
   * @throws \Exception
   */
  protected function readJson(string $type, string $filename, IOInterface $io): array {
    $io->write('<info>Gathering ' . $type . ' from ' . $filename . '.</info>');
    try {
      $data = json_decode(file_get_contents($filename), TRUE, 512, JSON_THROW_ON_ERROR);
      $error = json_last_error();
    }
    catch (\JsonException) {
      $error = JSON_ERROR_SYNTAX;
    }
    if ($error !== 0) {
      $msg = match ($error) {
        JSON_ERROR_DEPTH => ' - Maximum stack depth exceeded',
        JSON_ERROR_STATE_MISMATCH => ' - Underflow or the modes mismatch',
        JSON_ERROR_CTRL_CHAR => ' - Unexpected control character found',
        JSON_ERROR_SYNTAX => ' - Syntax error, malformed JSON',
        JSON_ERROR_UTF8 => ' - Malformed UTF-8 characters, possibly incorrectly encoded',
        default => ' - Unknown error',
      };
      throw new \Exception('There was an error in the supplied file:' . $msg);
    }
    return $data;
  }

}
